﻿using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.CSharp;

namespace DeclsToDLL
{
    public partial class frmMain : Form
    {
        public enum DataType
        {
            Void,
            Integer,
            Float,
            String,
            IntPtr
        }

        public class DeclsFunc
        {
            public string name;
            public string entryPoint;
            public DataType type = DataType.Void;

            public Dictionary<string, DataType> parameters = new Dictionary<string, DataType>();

        }

        public Dictionary<string, List<DeclsFunc>> libs = new Dictionary<string, List<DeclsFunc>>();

        public frmMain()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = @"*.decls file|*.decls";
            if (ofd.ShowDialog(this) == DialogResult.OK)
            {
                StreamReader sr = File.OpenText(ofd.FileName);
                string[] data = sr.ReadToEnd().Split(new[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);
                string lib = "";
                foreach (string s in data)
                {
                    if (s.Trim().StartsWith(";")) continue;
                    if (s.Trim().StartsWith(".lib"))
                    {
                        string[] data1 = s.Trim().Split(new[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                        lib = data1[data1.Length - 1];
                        if (!libs.ContainsKey(lib))
                        {
                            libs.Add(lib, new List<DeclsFunc>());
                        }
                        continue;
                    }

                    DeclsFunc func = new DeclsFunc();
                    func.name = s.Substring(0, s.IndexOf('('));

                    DataType fnType = DataType.Void;
                    string fnName = func.name.Trim();
                    if (fnName.EndsWith("#"))
                    {
                        fnName = fnName.Substring(0, fnName.Length - 1);
                        fnType = DataType.Float;
                    }
                    else if (fnName.EndsWith("%"))
                    {
                        fnName = fnName.Substring(0, fnName.Length - 1);
                        fnType = DataType.Integer;
                    }
                    else if (fnName.EndsWith("$"))
                    {
                        fnName = fnName.Substring(0, fnName.Length - 1);
                        fnType = DataType.String;
                    }
                    else if (fnName.EndsWith("*"))
                    {
                        fnName = fnName.Substring(0, fnName.Length - 1);
                        fnType = DataType.IntPtr;
                    }

                    func.name = fnName.Replace("_","");
                    func.type = fnType;
                    func.entryPoint = s.Substring(s.IndexOf(':') + 1);

                    if (func.entryPoint.StartsWith("\"_"))
                    {
                        func.entryPoint = "\"" + func.entryPoint.Substring(2);
                    }

                    if (func.entryPoint.Contains("@"))
                    {
                        func.entryPoint = func.entryPoint.Substring(0, 
                            func.entryPoint.IndexOf("@")) + "\"";
                    }

                    string pars = s.Substring(s.IndexOf('(') + 1);
                    pars = pars.Substring(0, pars.IndexOf(')'));

                    string[] parameters = pars.Split(new[] { ',', ' ' }, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string parameter in parameters)
                    {
                        DataType parType = DataType.Integer;
                        string parName = parameter.Trim();
                        if (parameter.EndsWith("#"))
                        {
                            parName = parName.Substring(0, parName.Length - 1);
                            parType = DataType.Float;
                        }
                        else if (parameter.EndsWith("%"))
                        {
                            parName = parName.Substring(0, parName.Length - 1);
                            parType = DataType.Integer;
                        }
                        else if (parameter.EndsWith("$"))
                        {
                            parName = parName.Substring(0, parName.Length - 1);
                            parType = DataType.String;
                        }
                        else if (parameter.EndsWith("*"))
                        {
                            parName = parName.Substring(0, parName.Length - 1);
                            parType = DataType.IntPtr;
                        }

                        func.parameters.Add(parName, parType);
                    }
                    libs[lib].Add(func);
                }

            }

            treeView1.Nodes.Clear();
            foreach (KeyValuePair<string, List<DeclsFunc>> lib in libs)
            {
                TreeNode libNode = new TreeNode(lib.Key);
                treeView1.Nodes.Add(libNode);
                foreach (DeclsFunc func in lib.Value)
                {
                    TreeNode funcNode = new TreeNode(func.type + " " + func.name + " - " + func.entryPoint);
                    libNode.Nodes.Add(funcNode);
                    foreach (var parameter in func.parameters)
                    {
                        TreeNode parNode = new TreeNode(parameter.Key + " - " + parameter.Value);
                        funcNode.Nodes.Add(parNode);
                    }
                }
            }

        }

        private void button2_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.Filter = @".Net Class Librarry|*.dll";
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("using System;");
                sb.AppendLine("using System.Runtime.InteropServices;");
                sb.AppendLine("namespace " + Path.GetFileNameWithoutExtension(saveFileDialog.FileName));
                sb.AppendLine("{");
                foreach (KeyValuePair<string, List<DeclsFunc>> lib in libs)
                {
                    string className = lib.Key.Trim().Replace("\"", "");
                    className = className.Substring(0, className.IndexOf("."));
                    sb.AppendLine("  public class " + className);
                    sb.AppendLine("  {");
                    foreach (DeclsFunc func in lib.Value)
                    {
                        sb.AppendLine("    [DllImport(" + lib.Key + ", EntryPoint = " + func.entryPoint +
                                      ", CharSet = CharSet.Auto)]");
                        string funcString = "    public static extern ";
                        switch (func.type)
                        {
                            case DataType.Void:
                                funcString += "void ";
                                break;
                            case DataType.Float:
                                funcString += "float ";
                                break;
                            case DataType.Integer:
                                funcString += "int ";
                                break;
                            case DataType.String:
                                funcString += "string ";
                                break;
                            case DataType.IntPtr:
                                funcString += "IntPtr ";
                                break;
                        }

                        funcString += func.name + "(";

                        if (func.parameters.Count > 0)
                        {
                            List<string> parameters = new List<string>();

                            foreach (KeyValuePair<string, DataType> parameter in func.parameters)
                            {
                                string par = "";

                                switch (parameter.Value)
                                {
                                    case DataType.Void:
                                        par += "void ";
                                        break;
                                    case DataType.Float:
                                        par += "float ";
                                        break;
                                    case DataType.Integer:
                                        par += "int ";
                                        break;
                                    case DataType.String:
                                        par += "string ";
                                        break;
                                    case DataType.IntPtr:
                                        par += "IntPtr ";
                                        break;
                                }
                                parameters.Add(par + parameter.Key);
                            }
                            funcString += string.Join(", ", parameters.ToArray());
                        }
                        sb.AppendLine(funcString + ");");
                        sb.AppendLine("");
                    }
                    sb.AppendLine("  }");
                }

                sb.AppendLine("}");

                // Настройки компиляции 
                Dictionary<string, string> providerOptions = new Dictionary<string, string> 
                { 
                    {"CompilerVersion", "v2.0"} 
                };
                CSharpCodeProvider provider = new CSharpCodeProvider(providerOptions);

                CompilerParameters compilerParams =
                    new CompilerParameters
                        {
                            OutputAssembly = saveFileDialog.FileName
                        };

                // Компиляция 
                CompilerResults results = provider.CompileAssemblyFromSource(compilerParams, sb.ToString());

                File.WriteAllText(saveFileDialog.FileName+".cs", sb.ToString());

                // Выводим информацию об ошибках 
                Console.WriteLine("Number of Errors: {0}", results.Errors.Count);
                if (results.Errors.Count>0)
                {
                    string errors = "";
                    foreach (CompilerError err in results.Errors)
                    {
                        errors+= err.ErrorText + "\n";
                    }
                    MessageBox.Show(errors);
                }
                else
                {
                    MessageBox.Show("Успешно!");
                }

                //File.WriteAllText(saveFileDialog.FileName, sb.ToString());
            }
        }

    }
}
